-release with major version 6."
	 *
	 * So if this function does not return a semver parseable version, then
	 * it must be one of these symbolic versions.
	 *
	 * The recommended way to resolve the symbolic versions 'latest',
	 * '5.x', or '6.x' into their current concrete values is to query the GraphQL
	 * API like this:
	 *
	 * ```
	 * query { release(version: "5.x") { version } }
	 * ```
	 *
	 * @since 4.0.0
	 * @see FontAwesome::latest_version()
	 * @see FontAwesome::releases_refreshed_at()
	 * @throws ConfigCorruptionException
	 * @return string|null null if no version has yet been saved in the options
	 * in the db. Otherwise, 5.x, or 6.x, or a semantic version string.
	 */
	public function version() {
		$options = $this->options();
		$this->validate_options( $options );

		return $options['version'];
	}

	/**
	 * Returns the currently configured kit token, if the plugin is currently
	 * configured to load a kit.
	 *
	 * @since 4.0.0
	 * @throws ConfigCorruptionException
	 * @return string|null kit token if present, or null
	 */
	public function kit_token() {
		$options = $this->options();
		$this->validate_options( $options );

		return isset( $options['kitToken'] ) ? $options['kitToken'] : null;
	}

	/**
	 * Indicates whether Font Awesome is being loaded with version 4 compatibility.
	 *
	 * Its result is valid only after the `font_awesome_enqueued` has been triggered.
	 *
	 * @since 4.0.0
	 * @throws ConfigCorruptionException
	 * @deprecated
	 *
	 * @return boolean
	 */
	public function v4_compatibility() {
		return $this->compatibility();
	}

	/**
	 * Indicates whether Font Awesome is being loaded with older version compatibility.
	 *
	 * Its result is valid only after the `font_awesome_enqueued` has been triggered.
	 *
	 * @since 4.1.0
	 * @throws ConfigCorruptionException
	 *
	 * @return boolean
	 */
	public function compatibility() {
		$options = $this->options();
		$this->validate_options( $options );
		return $options['compat'];
	}

	/**
	 * Indicates whether Font Awesome is being loaded with support for pseudo-elements.
	 *
	 * Its results are only valid after the `font_awesome_enqueued` action has been triggered.
	 *
	 * There are known performance problems with this SVG and pseudo-elements,
	 * but it is provided for added compatibility where pseudo-elements must be used.
	 *
	 * Always returns true if technology() === 'webfont', because pseudo-elements
	 * are always inherently supported by the CSS/Webfont technology.
	 *
	 * @since 4.0.0
	 * @link https://fontawesome.com/how-to-use/on-the-web/advanced/css-pseudo-elements CSS Pseudo-Elements and Font Awesome
	 * @throws ConfigCorruptionException
	 * @return boolean
	 */
	public function pseudo_elements() {
		$options = $this->options();
		$this->validate_options( $options );

		return $options['pseudoElements'];
	}

	/**
	 * Internal use only, not part of this plugin's public API.
	 *
	 * @internal
	 * @ignore
	 */
	protected function specified_preference_or_default( $preference, $default ) {
		return array_key_exists( 'value', $preference ) ? $preference['value'] : $default;
	}

	/**
	 * Enqueues the JavaScript bundle that is the React app for the admin
	 * settings page as well as the conflict detection reporter.
	 *
	 * The same bundle will be enqueued for both purposes. When enqueued, it
	 * must be configured to indicate which React components to mount in the DOM,
	 * which may be either, both, or neither.
	 *
	 * Internal use only, not part of this plugin's public API.
	 *
	 * @internal
	 * @ignore
	 */
	public function maybe_enqueue_admin_assets() {
		add_action(
			'admin_enqueue_scripts',
			function( $hook ) {
				$should_enable_icon_chooser = $this->should_icon_chooser_be_enabled( $hook );

				try {
					if ( $this->detecting_conflicts() || $hook === $this->screen_id || $should_enable_icon_chooser ) {
						$this->enqueue_admin_js_assets( $should_enable_icon_chooser );
					}

					if ( $hook === $this->screen_id ) {
						$this->maybe_refresh_releases();

						wp_localize_script(
							self::ADMIN_RESOURCE_HANDLE,
							self::ADMIN_RESOURCE_LOCALIZATION_NAME,
							array_merge(
								$this->common_data_for_js_bundle(),
								array(
									'showAdmin'            => true,
									'onSettingsPage'       => true,
									'clientPreferences'    => $this->client_preferences(),
									'releases'             => array(
										'available'        => $this->release_provider()->versions(),
										'latest_version_5' => $this->latest_version_5(),
										'latest_version_6' => $this->latest_version_6(),
									),
									'pluginVersion'        => FontAwesome::PLUGIN_VERSION,
									'preferenceConflicts'  => $this->conflicts_by_option(),
									'v3DeprecationWarning' => $this->get_v3deprecation_warning_data(),
								)
							)
						);
					} elseif ( $should_enable_icon_chooser ) {
						wp_localize_script(
							self::ADMIN_RESOURCE_HANDLE,
							self::ADMIN_RESOURCE_LOCALIZATION_NAME,
							array_merge(
								$this->common_data_for_js_bundle(),
								array(
									'enableIconChooser' => true,
								)
							)
						);

						/**
						 * TODO: re-enable the possibility of integrating with TinyMCE
						 * even on pages where Gutenberg is also present.
						 * This is an initial fix for GitHub Issue: #133
						 * https://github.com/FortAwesome/wordpress-fontawesome/issues/133
						 *
						 * UPATE: Now that the bundles are building differently
						 * and external dependencies are working correctly, this
						 * is close to working (try loading a new post in Gutenberg
						 * with integration plugin-nu enabled). It requires disabling
						 * the dynamic import of the components style.css. In the
						 * case where we're already on Gutenberg page, it is not
						 * necessary to load it, so it should be easy to check for that.
						 *
						 * The remaining issue seems to be just determining into
						 * which editor the shortcode should be inserted. When
						 * running plugin-nu, activating the Icon Chooser from eith