etpack\Device_Detection;

require_once __DIR__ . '/functions.php';

/**
 * A class providing device properties detection.
 */
class User_Agent_Info {

	/**
	 * Provided or fetched User-Agent string.
	 *
	 * @var string
	 */
	public $useragent;

	/**
	 * A device group matched user agent, e.g. 'iphone'.
	 *
	 * @var string
	 */
	public $matched_agent;

	/**
	 * Stores whether is the iPhone tier of devices.
	 *
	 * @var bool
	 */
	public $isTierIphone;

	/**
	 * Stores whether the device can probably support Rich CSS, but JavaScript (jQuery) support is not assumed.
	 *
	 * @var bool
	 */
	public $isTierRichCss;

	/**
	 * Stores whether it is another mobile device, which cannot be assumed to support CSS or JS (eg, older BlackBerry, RAZR)
	 *
	 * @var bool
	 */
	public $isTierGenericMobile;

	/**
	 * Stores the device platform name
	 *
	 * @var null|string
	 */
	private $platform             = null;
	const PLATFORM_WINDOWS        = 'windows';
	const PLATFORM_IPHONE         = 'iphone';
	const PLATFORM_IPOD           = 'ipod';
	const PLATFORM_IPAD           = 'ipad';
	const PLATFORM_BLACKBERRY     = 'blackberry';
	const PLATFORM_BLACKBERRY_10  = 'blackberry_10';
	const PLATFORM_SYMBIAN        = 'symbian_series60';
	const PLATFORM_SYMBIAN_S40    = 'symbian_series40';
	const PLATFORM_J2ME_MIDP      = 'j2me_midp';
	const PLATFORM_ANDROID        = 'android';
	const PLATFORM_ANDROID_TABLET = 'android_tablet';
	const PLATFORM_FIREFOX_OS     = 'firefoxOS';

	/**
	 * A list of dumb-phone user agent parts.
	 *
	 * @var array
	 */
	public $dumb_agents = array(
		'nokia',
		'blackberry',
		'philips',
		'samsung',
		'sanyo',
		'sony',
		'panasonic',
		'webos',
		'ericsson',
		'alcatel',
		'palm',
		'windows ce',
		'opera mini',
		'series60',
		'series40',
		'au-mic,',
		'audiovox',
		'avantgo',
		'blazer',
		'danger',
		'docomo',
		'epoc',
		'ericy',
		'i-mode',
		'ipaq',
		'midp-',
		'mot-',
		'netfront',
		'nitro',
		'palmsource',
		'pocketpc',
		'portalmmm',
		'rover',
		'sie-',
		'symbian',
		'cldc-',
		'j2me',
		'smartphone',
		'up.browser',
		'up.link',
		'up.link',
		'vodafone/',
		'wap1.',
		'wap2.',
		'mobile',
		'googlebot-mobile',
	);

	/**
	 * The constructor.
	 *
	 * @param string $ua (Optional) User agent.
	 */
	public function __construct( $ua = '' ) {
		if ( $ua ) {
			$this->useragent = $ua;
		} else {
			if ( ! empty( $_SERVER['HTTP_USER_AGENT'] ) ) {
				$this->useragent = strtolower( wp_unslash( $_SERVER['HTTP_USER_AGENT'] ) ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized -- This class is all about validating.
			}
		}
	}

	/**
	 * This method detects the mobile User Agent name.
	 *
	 * @return string The matched User Agent name, false otherwise.
	 */
	public function get_mobile_user_agent_name() {
		if ( $this->is_chrome_for_iOS() ) { // Keep this check before the safari rule.
			return 'chrome-for-ios';
		} elseif ( $this->is_iphone_or_ipod( 'iphone-safari' ) ) {
			return 'iphone';
		} elseif ( $this->is_ipad( 'ipad-safari' ) ) {
			return 'ipad';
		} elseif ( $this->is_android_tablet() ) { // Keep this check before the android rule.
			return 'android_tablet';
		} elseif ( $this->is_android() ) {
			return 'android';
		} elseif ( $this->is_blackberry_10() ) {
			return 'blackberry_10';
		} elseif ( $this->is_blackbeberry() ) {
			return 'blackberry';
		} elseif ( $this->is_WindowsPhone7() ) {
			return 'win7';
		} elseif ( $this->is_windows_phone_8() ) {
			return 'winphone8';
		} elseif ( $this->is_opera_mini() ) {
			return 'opera-mini';
		} elseif ( $this->is_opera_mini_dumb() ) {
			return 'opera-mini-dumb';
		} elseif ( $this->is_opera_mobile() ) {
			return 'opera-mobi';
		} elseif ( $this->is_blackberry_tablet() ) {
			return 'blackberry_tablet';
		} elseif ( $this->is_kindle_fire() ) {
			return 'kindle-fire';
		} elseif ( $this->is_PalmWebOS() ) {
			return 'webos';
		} elseif ( $this->is_S60_OSSBrowser() ) {
			return 'series60';
		} elseif ( $this->is_firefox_os() ) {
			return 'firefoxOS';
		} elseif ( $this->is_firefox_mobile() ) {
			return 'firefox_mobile';
		} elseif ( $this->is_MaemoTablet() ) {
			return 'maemo';
		} elseif ( $this->is_MeeGo() ) {
			return 'meego';
		} elseif ( $this->is_TouchPad() ) {
			return 'hp_tablet';
		} elseif ( $this->is_facebook_for_iphone() ) {
			return 'facebook-for-iphone';
		} elseif ( $this->is_facebook_for_ipad() ) {
			return 'facebook-for-ipad';
		} elseif ( $this->is_twitter_for_iphone() ) {
			return 'twitter-for-iphone';
		} elseif ( $this->is_twitter_for_ipad() ) {
			return 'twitter-for-ipad';
		} elseif ( $this->is_wordpress_for_ios() ) {
			return 'ios-app';
		} elseif ( $this->is_iphone_or_ipod( 'iphone-not-safari' ) ) {
			return 'iphone-unknown';
		} elseif ( $this->is_ipad( 'ipad-not-safari' ) ) {
			return 'ipad-unknown';
		} elseif ( $this->is_Nintendo_3DS() ) {
			return 'nintendo-3ds';
		} else {
			$agent       = $this->useragent;
			$dumb_agents = $this->dumb_agents;
			foreach ( $dumb_agents as $dumb_agent ) {
				if ( false !== strpos( $agent, $dumb_agent ) ) {
					return $dumb_agent;
				}
			}
		}

		return false;
	}

	/**
	 * This method detects the mobile device's platform. All return strings are from the class constants.
	 * Note that this function returns the platform name, not the UA name/type. You should use a different function
	 * if you need to test the UA capabilites.
	 *
	 * @return string Na