'id'    => $form->get_id(),
                'title' => $form->get_name() . ' (#' . $form->get_id() . ')',
            ];
        }

        wp_send_json_success( $response );
    }

    /**
     * Create a form
     *
     * @return void
     */
    public function create_form() {
        check_ajax_referer( 'weforms' );

        $this->check_admin();

        $form_name = isset( $_POST['form_name'] ) ? sanitize_text_field( wp_unslash( $_POST['form_name'] ) ) : '';

        if ( empty( $form_name ) ) {
            wp_send_json_error( __( 'Please provide a form name', 'weforms' ) );
        }

        $form_id = weforms()->form->create( $form_name );

        if ( is_wp_error( $form_id ) ) {
            wp_send_json_error( $form_id->get_error_message() );
        }

        wp_send_json_success( [
            'form_id'   => $form_id,
            'form_name' => $form_name,
        ] );
    }

    /**
     * Delete a form
     *
     * @return void
     */
    public function delete_form() {
        check_ajax_referer( 'weforms' );

        $this->check_admin();

        $form_id = isset( $_POST['form_id'] ) ? intval( $_POST['form_id'] ) : 0;

        if ( !$form_id ) {
            wp_send_json_error( __( 'No form id provided!', 'weforms' ) );
        }

        weforms()->form->delete( $form_id );

        wp_send_json_success();
    }

    public function delete_form_bulk() {
        check_ajax_referer( 'weforms' );

        $this->check_admin();

        $form_ids = isset( $_POST['ids'] ) ? array_map( 'absint', $_POST['ids'] ) : [];

        if ( !$form_ids ) {
            wp_send_json_error( __( 'No form ids provided!', 'weforms' ) );
        }

        foreach ( $form_ids as $form_id ) {
            weforms()->form->delete( $form_id );
        }

        wp_send_json_success();
    }

    /**
     * Duplicate a form
     *
     * @return voiud
     */
    public function duplicate_form() {
        check_ajax_referer( 'weforms' );

        $this->check_admin();

        $form_id = isset( $_POST['form_id'] ) ? intval( $_POST['form_id'] ) : 0;

        $form           = weforms()->form->duplicate( $form_id );
        $form->settings = $form->get_settings();

        wp_send_json_success( $form );
    }

    /**
     * Create form from a template
     *
     * @return void
     */
    public function create_form_from_template() {
        check_ajax_referer( 'weforms' );

        $template = isset( $_REQUEST['template'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['template'] ) ) : '';

        $form_id = weforms()->templates->create( $template );

        if ( is_wp_error( $form_id ) ) {
            wp_send_json_error( __( 'Could not create the form', 'weforms' ) );
        }

        wp_send_json_success( [
            'id' => $form_id,
        ] );
    }

    /**
     * Save the weForms settings
     *
     * @return void
     */
    public function save_settings() {
        check_ajax_referer( 'weforms' );
        $this->check_admin();

        $requires_wpuf_update = false;
        $wpuf_update_array    = array();
        $settings             = isset( $_POST['settings'] ) ? (array) json_decode( wp_unslash( $_POST['settings'] ) ) : [];
        update_option( 'weforms_settings', $settings );

        // wpuf settings sync
        if ( isset( $settings['gmap_api'] ) ) {
            $requires_wpuf_update              = true;
            $wpuf_update_array['gmap_api_key'] = $settings['gmap_api'];
        }

        if ( isset( $settings['recaptcha'] ) ) {
            $requires_wpuf_update                   = true;
            $wpuf_update_array['recaptcha_public']  = $settings['recaptcha']->key;
            $wpuf_update_array['recaptcha_private'] = $settings['recaptcha']->secret;
            $wpuf_update_array['recaptcha_type']    = $settings['recaptcha']->type;
        }

        if ( isset( $settings['no_conflict'] ) ) {
            $requires_wpuf_update              = true;
            $wpuf_update_array['no_conflict']  = $settings['no_conflict'];
        }

        if ( isset( $settings['email_footer'] ) ) {
            $requires_wpuf_update              = true;
            $wpuf_update_array['email_footer'] = $settings['email_footer'];
        }

        if ( $requires_wpuf_update ) {
            $wpuf_settings = get_option( 'wpuf_general', [] );

            $wpuf_settings = array_merge( $wpuf_settings, $wpuf_update_array );
            update_option( 'wpuf_general', $wpuf_settings );
        }

        do_action( 'weforms_save_settings', $settings );

        $settings = apply_filters( 'weforms_after_save_settings', $settings );
        wp_send_json_success( $settings );
    }

    /**
     * Get the weForms Settings
     *
     * @return void
     */
    public function get_settings() {
        check_ajax_referer( 'weforms' );

        $this->check_admin();

        $settings = weforms_get_settings();
        // checking to prevent js error, will be removed in future
        if ( !isset( $settings['credit'] ) ) {
            $settings['credit'] = false;
        }

        if ( !isset( $settings['permission'] ) ) {
            $settings['permission'] = 'manage_options';
        }

        wp_send_json_success( $settings );
    }

    /**
     * Get all entries
     *
     * @return void
     */
    public function get_entries() {
        check_ajax_referer( 'weforms' );

        $this->check_admin();

        $form_id      = isset( $_REQUEST['id'] ) ? intval( $_REQUEST['id'] ) : 0;
        $current_page = isset( $_REQUEST['page'] ) ? intval( $_REQUEST['page'] ) : 1;
        $status       = isset( $_REQUEST['status'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['status'] ) ) : 'publish';
        $per_page     = 20;
        $offset       = ( $current_page - 1 ) * $per_page;

        if ( !$form_id ) {
            wp_send_json_error( __( 'No form id provided!', 'weforms' ) );
        }

        $entries = weforms_get_form_entries(
            $form_id, [
                'number' => $per_page,
                'offset' => $offset,
                'status' => $status,
            ]
          );

        $columns       = weforms_get_entry_columns( $form_id );
        $total_entries = weforms_count_form_entries( $form_id, $status );

        array_map(
            function ( $entry ) use ( $columns ) {
                $entry_id = $entry->id;
                $entry->fields = [];

                foreach ( $columns as $meta_key => $label ) {
                    if ( empty( $meta_key ) ) {
                        continue;
                    }
                    $value                    = weforms_get_entry_meta( $entry_id, $meta_key, true );
                    $entry->fields[ $meta_key ] = str_replace( WeForms::$field_separator, ' ', $value );
                }
            }, $entries
          );

        $entries = apply_filters( 'weforms_get_entries', $entries, $form_id );

        $response = [
            'columns'    => $columns,
            'entries'    => $entries,
            'form_title' => get_post_field( 'post_title', $form_id ),
            'pagination' => [
                'total'    => $total_entries,
                'per_page' => $per_page,
                'pages'    => ceil( $total_entries / $per_page ),
                'current'  => $current_page,
            ],
            'meta' => [
                'total'      => weforms_count_form_entries( $form_id ),
                'totalTrash' => weforms_count_form_entries( $form_id, 'trash' ),
            ],
        ];

        wp_send_json_success( $response );
    }

    /**
     * Get an entry details
     *
     * @return void
     */
    public function get_entry_detail() {
        check_ajax_referer( 'weforms' );

        $this->check_admin();

        $form_id  = isset( $_REQUEST['form_id'] ) ? intval( $_REQUEST['form_id'] ) : 0;
        $entry_id = isset( $_REQUEST['entry_id'] ) ? intval( $_REQUEST['entry_id'] ) : 0;

        $form           = weforms()->form->get( $form_id );
        $form_settings  = $form->get_settings();
        $entry          = $form->entries()->get( $entry_id );
        $fields         = $entry->get_fields();
        $metadata       = $entry->get_metadata();
        $payment        = $entry->get_payment_data();

        if ( isset( $payment->payment_data ) && is_serialized( $payment->payment_data ) ) {
            $payment->payment_data = unserialize( $payment->payment_data );
        }

        if ( false === $fields ) {
            wp_send_json_error( __( 'No form fields found!', 'weforms' ) );
        }

        if ( sizeof( $fields ) < 1 ) {
            $fields[] = [
                'label' => __( 'No form fields found!', 'weforms' ),
            ];
        }

        $has_empty          = false;
        $answers            = [];
        $respondentPoints   = isset( $form_settings['total_points'] ) ? floatval( $form_settings['total_points'] ) : 0;
        $fields_formatted   = array();
        foreach ( $fields as $key => $field ) {
            if ( $form_settings['quiz_form'] == 'yes' ) {
                $selectedAnswers    = isset( $field['selected_answers'] ) ? $field['selected_answers'] : '';
                $givenAnswer        = isset( $field['value'] ) ? $field['value'] : '';
                $options            = isset( $field['options'] ) ? $field['options'] : '';
                $template           = $field['template'];
                $fieldPoints        = isset( $field['points'] ) ? floatval( $field['points'] ) : 0;

                if ( $template == 'radio_field' || $template == 'dropdown_field' ) {
                    $answers[$field['name']] = true;
                    if ( empty( $givenAnswer ) ) {
                        $answers[$field['name']] = false;
                        $respondentPoints -= $fieldPoints;
                    } else {
                        foreach ( $options as $key => $value ) {
                            if ( $givenAnswer == $value ) {
                                if ( $key != $selectedAnswers ) {
                                    $answers[$field['name']] = false;
                                    $respondentPoints -= $fieldPoints;
                                }
                            }
                        }
                    }
                } elseif ( $template == 'checkbox_field' || $template == 'multiple_select' ) {
                    $answers[$field['name']] = true;
                    $userAnswer              = [];

                    foreach ( $options as $key => $value ) {
                        foreach ( $givenAnswer as $answer ) {
                            if ( $value == $answer ) {
                                $userAnswer[] = $key;
                            }
                        }
                    }

                    $userAnswer   = implode( '|', $userAnswer );
                    $rightAnswers = implode( '|', $selectedAnswers );

                    if ( $userAnswer != $rightAnswers || empty( $userAnswer ) ) {
                        $answers[$field['name']] = false;
                        $respondentPoints -= $fieldPoints;
                    }
                }
            } elseif ( empty( $field['value'] ) ) {
                $has_empty      = true;
                break;
            } else {
                $field = WeForms_Form_Entry_Manager::format_entry_value( $field );
                array_push( $fields_formatted, $field );
            }
        }
        $response = [
            'form_fields'       => $fields_formatted,
            'form_settings'     => $form_settings,
            'meta_data'         => $metadata,
            'payment_data'      => $payment,
            'has_empty'         => $has_empty,
            'respondent_points' => $respondentPoints,
            'answers'           => $answers,
        ];


        wp_send_json_success( $response );
    }

    /**
     * Trash an entry
     *
     * @return void
     */
    public function trash_entry() {
        check_ajax_referer( 'weforms' );

        $this->check_admin();

        $entry_id = isset( $_REQUEST['entry_id'] ) ? intval( $_REQUEST['entry_id'] ) : 0;

        weforms_change_entry_status( $entry_id, 'trash' );
        wp_send_json_success();
    }

    /**
     * Trash an entry
     *
     * @return void
     */
    public function delete_entry() {
        check_ajax_referer( 'weforms' );

        $this->check_admin();

        $entry_id = isset( $_REQUEST['entry_id'] ) ? intval( $_REQUEST['entry_id'] ) : 0;

        weforms_delete_entry( $entry_id );

        wp_send_json_success();
    }

    /**
     * Restore Entry
     *
     * @return void
     */
    public function restore_entry() {
        check_ajax_referer( 'weforms' );

        $this->check_admin();

        $entry_id = isset( $_REQUEST['entry_id'] ) ? intval( $_REQUEST['entry_id'] ) : 0;

        weforms_change_entry_status( $entry_id, 'publish' );
        wp_send_json_success();
    }

    /**
     * Bulk trash entries
     *
     * @return void
     */
    public function bulk_delete_entry() {
        check_ajax_referer( 'weforms' );

        $this->check_admin();

        $entry_ids = isset( $_POST['ids'] ) ? array_map( 'absint', $_POST['ids'] ) : array();
        $permanent = isset( $_POST['permanent'] ) && ( sanitize_text_field( wp_unslash ( $_POST['permanent'] ) ) ) ? true : false;

        if ( !$entry_ids ) {
            wp_send_json_error( __( 'No entry ids provided!', 'weforms' ) );
        }

        foreach ( $entry_ids as $entry_id ) {
            if ( $permanent ) {
                weforms_delete_entry( $entry_id );
            } else {
                weforms_change_entry_status( $entry_id, 'trash' );
            }
        }

        wp_send_json_success();
    }

    /**
     * Bulk trash entries
     *
     * @return void
     */
    public function bulk_restore_entry() {
        check_ajax_referer( 'weforms' );

        $this->check_admin();

        $entry_ids = isset( $_POST['ids'] ) ? array_map( 'absint', $_POST['ids'] ) : [];

        if ( !$entry_ids ) {
            wp_send_json_error( __( 'No entry ids provided!', 'weforms' ) );
        }

        foreach ( $entry_ids as $entry_id ) {
            weforms_change_entry_status( $entry_id, 'publish' );
        }

        wp_send_json_success();
    }

    /**
     * Handle the frontend submission
     *
     * @return void
     */
    public function handle_frontend_submission() {
        check_ajax_referer( 'wpuf_form_add' );

        $form_id       = isset( $_POST['form_id'] ) ? intval( $_POST['form_id'] ) : 0;
        $page_id       = isset( $_POST['page_id'] ) ? intval( $_POST['page_id'] ) : 0;

        $form          = weforms()->form->get( $form_id );
        $form_settings = $form->get_settings();
        $form_fields   = $form->get_fields();
        $entry_fields  = $form->prepare_entries();
        $form_entries  = weforms_get_form_entries( $form_id, [ 'number'  => '', 'offset'  => '' ] );

        if ( $form_fields && count( $form_entries ) && count( $entry_fields ) ) {
            foreach ( $entry_fields as $field_key => $field_value ) {
                $duplicate_check = false;
                $field_label     = 'This';

                foreach ( $form_fields as $form_field ) {
                    if ( in_array( $form_field['template'], [ 'text_field', 'website_url', 'numeric_text_field', 'email_address' ] ) && $form_field['name'] == $field_key && isset( $form_field['duplicate'] ) && 'no' == $form_field['duplicate'] ) {
                        $duplicate_check = true;
                        $field_label     = $form_field['label'];
                    }
                }

                if ( $duplicate_check ) {
                    foreach ( $form_entries as $entry ) {
                        $existing = weforms_get_entry_meta( $entry->id, $field_key, true );

                        if ( $existing && $field_value == $existing ) {
                            wp_send_json( [
                                'success' => false,
                                'error'   => sprintf( __( '"%s" field requires a unique entry and "%s" has already been used.', 'weforms' ), $field_label, $field_value ),
                            ] );
                        }
                    }
                }
            }
        }

        if ( !$form_fields ) {
            wp_send_json( [
                'success'     => false,
                'error'       => __( 'No form field was found.', 'weforms' ),
            ] );
        }

        if ( $form->has_field( 'recaptcha' ) ) {
            $settings     = weforms_get_settings( 'recaptcha' );
            $type         = isset( $settings->type ) ? $settings->type : '';
            $secret       = isset( $settings->secret ) ? $settings->secret : '';
            if( $type == 'v3' ) {
                $this->validate_reCaptchav3( $secret );
             } else {
                $this->validate_reCaptcha();
            }
        }

        // vaidate submission
        $this->validate_submission( $entry_fields, $form, $form_settings, $form_fields );

        $entry_fields = apply_filters( 'weforms_before_entry_submission', $entry_fields, $form, $form_settings, $form_fields );

        $entry_id = 1;
        $global_settings = weforms_get_settings();
        if ( empty( $form_settings['after_submission'] ) ) {
            $entry_id = weforms_insert_entry( [
                'form_id' => $form_id,
            ], $entry_fields );
            if ( is_wp_error( $entry_id ) ) {
                wp_send_json( [
                    'success' => false,
                    'error'   => $entry_id->get_error_message(),
                ] );
            }
            // Fire a hook for integration
            do_action( 'weforms_entry_submission', $entry_id, $form_id, $page_id, $form_settings );
            $notification = new WeForms_Notification( [
                'form_id'  => $form_id,
                'page_id'  => $page_id,
                'entry_id' => $entry_id,
            ] );
            $notification->send_notifications();
        }
        // redirect URL
        $show_message = false;
        $redirect_to = false;
        if ( $form_settings['redirect_to'] == 'page' ) {
            $redirect_to = get_permalink( $form_settings['page_id'] );
        } elseif ( $form_settings['redirect_to'] == 'url' ) {
            $redirect_to = $form_settings['url'];
        } elseif ( $form_settings['redirect_to'] == 'same' ) {
            $show_message = true;
        } else {
            $show_message = true;
        }
        $field_search = $field_replace = [];
        foreach ( $form_fields as $r_field ) {
            $field_search[] = '{' . $r_field['name'] . '}';
            if ( $r_field['template'] == 'name_field' ) {
                $field_replace[] = implode( ' ', explode( '|', $entry_fields[ $r_field['name'] ] ) );
            } else if ( $r_field['template'] == 'address_field' ) {
                $field_replace[] = implode( ', ', $entry_fields[ $r_field['name'] ] );
            } else {
                $field_replace[] = isset( $entry_fields[ $r_field['name'] ] ) ? $entry_fields[ $r_field['name'] ] : '';
            }
        }
        $message = str_replace( $field_search, $field_replace, $form_settings['message'] );
        // send the response
        $response = apply_filters( 'weforms_entry_submission_response', [
            'success'   => true,
            'redirect_to' => $redirect_to,
            'show_message' => $show_message,
            'message'   => $message,
            'data'    => $_POST,
            'form_id'   => $form_id,
            'entry_id'  => $entry_id,
            'entry_fields' =>$entry_fields,
        ] );
        weforms_clear_buffer();
        wp_send_json( $response );
        }

        function validate_reCaptchav3( $secret ) {
            check_ajax_referer( 'wpuf_form_add' );

            $post_data          = wp_unslash($_POST);
            $token              = $post_data['g-recaptcha-response'];
            $action             = $post_data['g-action'];
            $google_captcha_url = esc_url( 'https://www.google.com/recaptcha/api/siteverify' );

            $response = wp_remote_post( $google_captcha_url,
                array(
                    'method'      => 'POST',
                    'body'        => array(
                        'secret'   => $secret,
                        'response' => $token
                    )
                )
            );


            if ( is_wp_error( $response ) ) {
                wp_send_json( [
                    'success'     => false,
                    'error'       => __( 'reCAPTCHA validation failed', 'weforms' ),
                ] );
            } else {
                $api_response = json_decode( wp_remote_retrieve_body( $response ), true );
                if( $api_response["success"] == '1' && $api_response["action"] == $action  ) {
                    return true;
                } else {
                    wp_send_json( [
                        'success'     => false,
                        'error'       => __( 'reCAPTCHA validation failed', 'weforms' ),
                    ] );
                }
            }
        }
    /**
     * reCaptcha Validation
     *
     * @return void
     */
    function validate_reCaptcha() {
        check_ajax_referer( 'wpuf_form_add' );
        if ( class_exists( 'WPUF_ReCaptcha' ) ) {
            $recaptcha_class = 'WPUF_ReCaptcha';
        } else {
            if ( !function_exists( 'recaptcha_get_html' ) ) {
                require_once WEFORMS_INCLUDES . '/library/reCaptcha/recaptchalib.php';
            }

            require_once WEFORMS_INCLUDES . '/library/reCaptcha/recaptchalib_noCaptcha.php';
            $recaptcha_class = 'Weforms_ReCaptcha';
        }

        $invisible = isset( $_POST['g-recaptcha-response'] ) ? false : true;

        $recaptcha_settings = weforms_get_settings( 'recaptcha' );
        $secret             = isset( $recaptcha_settings->secret ) ? $recaptcha_settings->secret : '';

        if ( ! $invisible ) {
            $response           = null;
            $reCaptcha          = new $recaptcha_class( $secret );
            $remote_ADDR        = isset( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash(  $_SERVER['REMOTE_ADDR'] ) ) : '';
            $recaptcha_response = isset( $_POST['g-recaptcha-response'] ) ? sanitize_text_field(  wp_unslash(  $_POST['g-recaptcha-response'] ) ) : '';
            $resp = $reCaptcha->verifyResponse(
                $remote_ADDR,
                $recaptcha_response
            );

            if ( !$resp->success ) {
                wp_send_json( [
                    'success'     => false,
                    'error'       => __( 'reCAPTCHA validation failed', 'weforms' ),
                ] );
            }

		} else {

            $recap_challenge = isset( $_POST['recaptcha_challenge_field'] ) ? sanitize_text_field( wp_unslash( $_POST['recaptcha_challenge_field'] ) ) : '';
            $recap_response  = isset( $_POST['recaptcha_response_field'] ) ? sanitize_text_field( wp_unslash( $_POST['recaptcha_response_field'] ) ) : '';
            $resp            = recaptcha_check_answer( $secret, $remote_ADDR, $recap_challenge, $recap_response );

            if ( !$resp->is_valid ) {
                ob_clean();

                wp_send_json( [
                    'success'     => false,
                    'error'       => __( 'reCAPTCHA validation failed', 'weforms' ),
                ] );
            }
        }
    }

    /**
     * Validate submission
     *
     * @param array  $entry_fields
     * @param object $form
     * @param array  $form_settings
     * @param array  $form_fields
     *
     * @return bool|json
     */
    public function validate_submission( $entry_fields, $form, $form_settings, $form_fields ) {
        foreach ( $form_fields as $key => $field ) {

            //skip custom html field as it is not saved
            if ( 'custom_html' == $field['name'] ) {
                continue;
            }

            //skip recaptcha field as it is not saved
            if ( 'recaptcha' == $field['name'] ) {
                continue;
            }

            $value = $entry_fields[ $field['name'] ];

            if ( 'single_product' === $field['template'] ) {
                if ( !$value ) {
                    $value = [];
                }

                $value['price']    = isset( $value['price'] ) ? floatval( $value['price'] ) : 0;
                $value['quantity'] = isset( $value['quantity'] ) ? floatval( $value['quantity'] ) : 0;
                $quantity          = isset( $field['quantity'] ) ? $field['quantity'] : [];
                $price             = isset( $field['price'] ) ? $field['price'] : [];

                if ( isset( $price['is_flexible'] ) && $price['is_flexible'] ) {
                    $min = isset( $price['min'] ) ? floatval( $price['min'] ) : 0;
                    $max = isset( $price['max'] ) ? floatval( $price['max'] ) : 0;

                    if ( $value['price'] < $min ) {
                        wp_send_json( [
                            'success'     => false,
                            'error'       => __( sprintf(
                                '%s price must be equal or greater than %s',
                                $field['weforms'],
                                $min ),
                            'weforms' ),
                        ] );
                    }

                    if ( $max && $value['price'] > $max ) {
                        wp_send_json( [
                            'success'     => false,
                            'error'       => __( sprintf(
                                '%s price must be equal or less than %s',
                                $field['weforms'],
                                $max ),
                            'weforms' ),
                        ] );
                    }
                }

                if ( isset( $quantity['status'] ) && $quantity['status'] ) {
                    $min = isset( $quantity['min'] ) ? floatval( $quantity['min'] ) : 0;
                    $max = isset( $quantity['max'] ) ? floatval( $quantity['max'] ) : 0;

                    if ( $value['quantity'] < $min ) {
                        wp_send_json( [
                            'success'     => false,
                            'error'       => __( sprintf(
                                '%s quantity must be equal or greater than %s',
                                $field['weforms'],
                                $min ),
                            'weforms' ),
                        ] );
                    }

                    if ( $max && $value['quantity'] > $max ) {
                        wp_send_json( [
                            'success'     => false,
                            'error'       => __( sprintf(
                                '%s quantity must be equal or less than %s',
                                $field['weforms'],
                                $max ),
                            'weforms' ),
                        ] );
                    }
                }
            }
        }
    }

    public static function prepare_meta_fields( $meta_vars ) {
        check_ajax_referer( 'wpuf_form_add